/***************************************************************************
 *   Copyright (C) 2004 by Andr Stein                                     *
 *   stone@steinsoft.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "sglslprogram.h"
#include "sglslshader.h"
#include "sglslextension.h"
#include <fstream>
#include <iostream>

using namespace std;

sglslProgram::sglslProgram()
: handle (0)
{
}

sglslProgram::sglslProgram(const char* vsFilename, const char* fsFilename)
: handle(0)
{
	createFromFile(vsFilename, fsFilename);
}

sglslProgram::~sglslProgram()
{
	//at least we want to clean up..
	destroy();
}

void sglslProgram::createFromFile(const char* vsFilename, const char* fsFilename)
{
	//pass everything to the create(..) function
	//using loadShader we actually load contents from the given
	//file and make life easier to everyone...
	if (fsFilename)
		create(loadShader(vsFilename).c_str(), loadShader(fsFilename).c_str());
	else
		create(loadShader(vsFilename).c_str());
}


void sglslProgram::create(const char* vsSource, const char* fsSource)
{
	destroy();
	
	/** first create
	    program object
	 **/
	 
	handle = pglCreateProgramObjectARB();

	//create the shaders..
	sglslShader vertexShader;
	sglslShader fragmentShader;
	
	
	vertexShader.createVertexShader(vsSource);
	
	if (fsSource)
		fragmentShader.createFragmentShader(fsSource);
		
	
	//attach shaders to program
	pglAttachObjectARB(handle, vertexShader.getHandle());
	
	if (fsSource)
		pglAttachObjectARB(handle, fragmentShader.getHandle());
		
	
	//now link everything to make it ready for use
	pglLinkProgramARB(handle);
}

void sglslProgram::destroy()
{
	if (handle)
	{
		pglDeleteObjectARB(handle);
		//reset		
		handle = 0;
	}
}

const string sglslProgram::loadShader(const char* filename)
{
	ifstream file(filename);
	char* buffer;
	
	if(!file)
	{
		cerr << filename << " shader could not be loaded." << endl;
		return string(0);
	}
	
	//get size of file to be loaded..
	file.seekg(0,ios::end);
	int size = file.tellg();
	
	//load into buffer
	buffer = new char[size + 1];
	file.seekg(0, ios::beg);
	file.read(buffer, size);
	//the end
	buffer[size] = '\0';
	file.close();
	
	string contents(buffer);
	delete[] buffer;
	
	return contents;
}

void sglslProgram::bind()
{
	pglUseProgramObjectARB(handle);
}

void sglslProgram::setUniform(const char* name, int size, float* matrix, bool transpose)
{
	switch (size)
	{
	case 2:
		pglUniformMatrix2fvARB(pglGetUniformLocationARB(handle,name), 0, transpose, matrix);
		break;
	case 3:
		pglUniformMatrix3fvARB(pglGetUniformLocationARB(handle,name), 0, transpose, matrix);
		break;
	case 4: 
		pglUniformMatrix4fvARB(pglGetUniformLocationARB(handle,name), 0, transpose, matrix);
		break;
	}
}